@@ -3,6 +3,8 @@
   ## gc is necessary to collect external pointers who have not yet been collected
   ## that have finalizers pointing to the soon-to-unloaded dll
   gc(FALSE)
-  .Call("dbarts_finalize")
-  library.dynam.unload("dbarts", libpath)
+  if (is.loaded("dbarts_finalize", PACKAGE = "dbarts")) {
+    .Call("dbarts_finalize")
+    library.dynam.unload("dbarts", libpath)
+  }
 }
