@@ -5,7 +5,7 @@
 #include <R.h>
 
 #define MAX_BUFFER_LENGTH 8192
-void ext_throwError(const char* format, ...)
+NORETURN void ext_throwError(const char* format, ...)
 {
   char buffer[MAX_BUFFER_LENGTH];
   
